@@ -1,75 +1,96 @@
 import os
 import socket
 import struct
-import subprocess
 from time import sleep
-from typing import Any, Callable, Generator, Optional
+from typing import Any, Callable, Generator, Optional, Union
 
 import cv2
 import numpy as np
+from adbutils import AdbDevice, AdbError, Network, _AdbStreamConnection, adb
 from av.codec import CodecContext
 
+from .const import EVENT_FRAME, EVENT_INIT, LOCK_SCREEN_ORIENTATION_UNLOCKED
 from .control import ControlSender
 
 
 class Client:
     def __init__(
         self,
-        max_width=0,
-        bitrate=8000000,
-        max_fps=0,
-        adb_path="/usr/local/bin/adb",
-        ip="127.0.0.1",
-        port=8081,
-        flip=False,
-        block_frame=False,
+        device: Optional[Union[AdbDevice, str]] = None,
+        max_width: int = 0,
+        bitrate: int = 8000000,
+        max_fps: int = 0,
+        flip: bool = False,
+        block_frame: bool = False,
+        stay_awake: bool = False,
+        lock_screen_orientation: int = LOCK_SCREEN_ORIENTATION_UNLOCKED,
     ):
         """
         Create a scrcpy client, this client won't be started until you call the start function
+        :param device: Android device, select first one if none, from serial if str
         :param max_width: frame width that will be broadcast from android server
         :param bitrate: bitrate
         :param max_fps: 0 means not max fps. supported after android 10
-        :param adb_path: adb path
-        :param ip: android server IP
-        :param port: android server port
         :param flip: flip the video
         :param block_frame: only return nonempty frames, may block cv2 render thread
+        :param stay_awake: keep Android device awake
+        :param lock_screen_orientation: lock screen orientation
         """
 
-        self.ip = ip
-        self.port = port
+        if device is None:
+            device = adb.devices()[0]
+        elif isinstance(device, str):
+            device = adb.device(serial=device)
+
+        self.device = device
         self.listeners = dict(frame=[], init=[])
+
+        # User accessible
         self.last_frame = None
-        self.video_socket = None
-        self.control_socket = None
         self.resolution = None
-        self.adb_path = adb_path
-        self.flip = flip
         self.device_name = None
         self.control = ControlSender(self)
+
+        # Params
+        self.flip = flip
         self.max_width = max_width
         self.bitrate = bitrate
         self.max_fps = max_fps
         self.block_frame = block_frame
+        self.stay_awake = stay_awake
+        self.lock_screen_orientation = lock_screen_orientation
+
+        # Need to destroy
+        self.alive = False
+        self.server_stream: Optional[_AdbStreamConnection] = None
+        self.video_socket: Optional[socket.socket] = None
+        self.control_socket: Optional[socket.socket] = None
 
     def init_server_connection(self):
         """
         Connect to android server, there will be two sockets, video and control socket.
         This method will set: video_socket, control_socket, resolution variables
         """
-
-        self.video_socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
-        self.video_socket.connect((self.ip, self.port))
+        for _ in range(30):
+            try:
+                self.video_socket = self.device.create_connection(
+                    Network.LOCAL_ABSTRACT, "scrcpy"
+                )
+                break
+            except AdbError:
+                sleep(0.1)
+                pass
+        else:
+            raise ConnectionError("Failed to connect scrcpy-server after 3 seconds")
 
         dummy_byte = self.video_socket.recv(1)
         if not len(dummy_byte):
             raise ConnectionError("Did not receive Dummy Byte!")
 
-        self.control_socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
-        self.control_socket.connect((self.ip, self.port))
-
+        self.control_socket = self.device.create_connection(
+            Network.LOCAL_ABSTRACT, "scrcpy"
+        )
         self.device_name = self.video_socket.recv(64).decode("utf-8")
-
         if not len(self.device_name):
             raise ConnectionError("Did not receive Device Name!")
 
@@ -81,47 +102,34 @@ class Client:
         """
         Deploy server to android device
         """
-
-        if not os.path.exists(self.adb_path):
-            raise FileNotFoundError(
-                "Couldn't find ADB at path ADB_bin: " + str(self.adb_path)
-            )
-
         server_root = os.path.abspath(os.path.dirname(__file__))
         server_file_path = server_root + "/scrcpy-server.jar"
-        adb_push = subprocess.Popen(
-            [self.adb_path, "push", server_file_path, "/data/local/tmp/"],
-            stdout=subprocess.PIPE,
-            stderr=subprocess.PIPE,
-            cwd=server_root,
-        )
-        adb_push_comm = "".join(
-            [x.decode("utf-8") for x in adb_push.communicate() if x is not None]
-        )
-
-        if "error" in adb_push_comm:
-            raise ConnectionError("Is your device/emulator visible to ADB?")
-
-        subprocess.Popen(
+        self.device.push(server_file_path, "/data/local/tmp/")
+        self.server_stream = self.device.shell(
             [
-                self.adb_path,
-                "shell",
                 "CLASSPATH=/data/local/tmp/scrcpy-server.jar",
                 "app_process",
                 "/",
-                "com.genymobile.scrcpy.Server 1.12.1 {} {} {} true - false true".format(
-                    self.max_width, self.bitrate, self.max_fps
-                ),
+                "com.genymobile.scrcpy.Server",
+                "1.18",
+                "info",
+                f"{self.max_width}",
+                f"{self.bitrate}",
+                f"{self.max_fps}",
+                f"{self.lock_screen_orientation}",
+                "true",
+                "-",
+                "false",
+                "true",
+                "0",
+                "false",
+                "true" if self.stay_awake else "false",
+                "-",
+                "-",
+                "false",
             ],
-            cwd=server_root,
+            stream=True,
         )
-        sleep(1)
-
-        subprocess.Popen(
-            [self.adb_path, "forward", f"tcp:{self.port}", "localabstract:scrcpy"],
-            cwd=server_root,
-        ).wait()
-        sleep(1)
 
     def start(self) -> None:
         """
@@ -129,22 +137,23 @@ class Client:
         """
         self.deploy_server()
         self.init_server_connection()
-        self.__send_to_listeners("init")
+        self.__send_to_listeners(EVENT_INIT)
+        self.alive = True
+        self.__stream_loop()
 
-        for i in self.__stream_generator():
-            if i is not None:
-                self.last_frame = i
-                self.resolution = (i.shape[1], i.shape[0])
-            self.__send_to_listeners("frame", i)
+    def stop(self) -> None:
+        self.alive = False
+        if self.server_stream is not None:
+            self.server_stream.close()
+        if self.control_socket is not None:
+            self.server_stream.close()
+        if self.video_socket is not None:
+            self.video_socket.close()
 
-    def __stream_generator(self) -> Generator[Optional[np.ndarray], None, None]:
-        """
-        Parsing h264 stream to frames
-        :return: frames
-        """
+    def __stream_loop(self):
         codec = CodecContext.create("h264", "r")
 
-        while True:
+        while self.alive:
             try:
                 raw_h264 = self.video_socket.recv(0x10000)
                 packets = codec.parse(raw_h264)
@@ -154,10 +163,15 @@ class Client:
                         frame = frame.to_ndarray(format="bgr24")
                         if self.flip:
                             frame = cv2.flip(frame, 1)
-                        yield frame
+                        self.last_frame = frame
+                        self.resolution = (frame.shape[1], frame.shape[0])
+                        self.__send_to_listeners(EVENT_FRAME, frame)
             except BlockingIOError:
                 if not self.block_frame:
-                    yield None
+                    self.__send_to_listeners(EVENT_FRAME, None)
+            except OSError as e:  # Socket Closed
+                if self.alive:
+                    raise e
 
     def add_listener(self, cls: str, listener: Callable[..., Any]) -> None:
         """
